/*
  Dynamic Focus Handler for Arex 3 and UniQ
  Developed on 2019 02 21
  Author M.P. care of Datalogic S.p.A.
	
Ver. 1.0 :
 - The Rs-232 configuration is on file 'config.txt', and the GUI icons inside the 'Outcome' group box are shown.
 - The Z axis is moved to the new position only if the |newMeasure-oldMeasure|>=TOLERANCE.
 - Communication Error is reported only when the Distance Sensor doesn't reply to the command after TIMER seconds after that the script has sent a trigger.
 
 Ver. 1,1 : 
 - The name of the script has been changed to 'DynamicFocusHandlerArexUniQ'
 
 Ver. 1.2:
 - Correction to the script description.
 - Limited the log section of the GUI to 3 lines;
 - Added the classMainWindow.prototype.destinationOutOfRange method;
 
 Ver. 1.3
 - Unified Error Messages (Implementation Task #91129)
 - Modified the default LOG_LINES to 3 for screen resolution purposes
 - Modified default TOLERANCE to 0.5 for application purposes
*/

/***********************************************************
  Definitions of constants
***********************************************************/
// Works definition

// Internal messages
const AppTitle = "Datalogic Automation Standard Serial Protocol Manager";
const SEL_ERROR = "Unable to select the requested work";
const LOAD_ERROR = "Unable to load the work set!";
const ESC_ERROR = "Invalid command detected";
const ESC_WARNING = "Unsupported command detected";
const STR_ERROR = "Unable to set the requested string data";
const CNT_ERROR = "Unable to get the requested counter";
const BUSY_ERROR = "Command received while system busy!";

const savedPosition=undefined;
//Tracing
const TRACE = true;
const DISTANCE = "DISTANCE";
const TOLERANCE=0.50;    //tolerance expressed in millimiters;

var M, m, measureHeight=false, setUpSensorPhase=true, step=0, autofocusOngoing=false, lastCheck=false;

/***********************************************************
  Main function
***********************************************************/
function main()
{
  System.sigQueryStart.connect(onQueryStart);
  System.sigLaserStop.connect(onLaserStop);
  System.sigLaserStart.connect(onLaserStart);
  System.sigLaserEnd.connect(onLaserEnd);
  Axis.sigReachPosition.connect(onPositionReached);
  System["sigLaserError(int)"].connect(onLaserError);
  System.sigClose.connect(onClose);
  
  h_ver = System.version().split(" ")[0].slice(0,3);
  hDoc = new LaserDoc();
  h_port = new classPort();
  h_Window = new classMainWindow(h_port);
  initWindowEvents();
  h_Proto = new classProtocol();
  onbtnOpen();
  IoPort.setReady(true);
  
  m=Axis.getMinPosition(2);
  M=Axis.getMaxPosition(2);
  log("Z Axis enabled: "+Axis.isEnabled(2)+", Max: "+M+", Min: "+m);

  err="ERR:"+DISTANCE;
  if(err==System.getStringValue(DISTANCE))print("No distance reference value found on this laser");
  else {
      d = parseFloat(System.getStringValue(DISTANCE)).toFixed(2);
      print("The reference distance is present on this laser: "+d);
      h_Window.PBfocusStart.enable=true;
  }
  
  trace( "Std RS-232 Ready !" );
  h_Window.OK();
  h_Window.onIndexChanged();
  setupSensor(true);
}

/***********************************************************
  This functions associates button events to some functions
***********************************************************/
function initWindowEvents()
{
  h_Window.btn[0]["sigReleased"].connect(onbtnOpen);
  h_Window.btn[1]["sigReleased"].connect(onbtnClose);
}

/***********************************************************
  ComPort management functions
***********************************************************/
function onbtnOpen()
{
  System.collectGarbage();
  h_Proto.start( h_port, h_Window );
  trace("Protocol started!");
}

function onbtnClose()
{
  h_Proto.stop();
  trace("Protocol stopped!");
}

/***********************************************************
  Laser management functions
***********************************************************/
function onQueryStart()
{
  trace("onQueryStart");
  if (h_ver < 5.2) hDoc.preview();
  else h_Window.prv.preview(hDoc);
  if(!autofocusOngoing) h_Window.onFocusStart();
  else log("Received a START signal as the Autofocus process is ongoing");
}

function onMark() {
    	h_Window.PBtrigger.enable=false;
	h_Window.PBsaveDistance.enable=false;
	h_Window.PBfocusStart.enable=false;
    hDoc.execute();
}

function onLaserStop()
{
  trace("onLaserStop");
}

function onLaserStart()
{
  // TODO
}

function onLaserEnd()
{
    h_Window.PBtrigger.enable=true;
    h_Window.PBsaveDistance.enable=true;
    h_Window.PBfocusStart.enable=true;
    h_Window.prv.preview(hDoc);
}

function onPositionReached() 
{
    log("Position reached!");
    savedPosition=Axis.getPosition(2).toFixed(2);
    if(measureHeight) {
	measureHeight=false;
	log("Making a precision acquisition");
	    if (h_Proto.protocolSend("{0M}")) {
		print("sentMessage");
	    }	
    }
    else {
	print("End of second acquisition");
	System.collectGarbage();
	autofocusOngoing=false;
	lastCheck=true;
	if (h_Proto.protocolSend("{0M}")) {   //ensuring that after the second movement of the Axis, the right position has been reached
	    print("sentMessage");
	}
    }
    
}

function onLaserError(error)
{
  switch(error)
  {
    case System.DSP_IN_HANG:
      System.resetBoard();
      break;
    case System.DSP_ERROR_INIT:
      // DSp initialization failure
      break;
    // TODO
  }
}

function onClose()
{
  IoPort.setReady(false); 
  h_Window.close();
}
